/*
 *           Titolo: Studio delle prestazioni dell'algoritmo "INSERTION SORT"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 22/11/2016
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define N_MAX 4000000

int insertion_sort(int dvett[], int svett[], int dim_s) {
	int dim_d = 0;

	for(int i = 0; i < dim_s; i++) {
		int j = dim_d;
		while (j > 0 && dvett[j-1] > svett[i]) {
			dvett[j] = dvett[j-1];
			j--;
		}
		dvett[j] = svett[i];
		dim_d++;
	}
	return dim_d;
}

double misura_tempo(int dvett[], int svett[], int dim) {
	clock_t t_inizio, t_fine;
	clock_t ticks;

	t_inizio = clock();     // Tempo di CPU consumato dal processo fino a questo punto [tick]
	
	insertion_sort(dvett, svett, dim);
	
	t_fine = clock(); 		// Nuovo tempo di CPU consumato dal processo fino a questo punto [tick]
	
	ticks = t_fine - t_inizio;  // Intervallo di tempo tra i due istanti [tick]
	
	return 1000.0 * ticks / CLOCKS_PER_SEC;     // Conversione dell'intervallo di tempo in ms
	
}

void inizializza_casuale(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = rand(); }
void inizializza_crescente(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = i+1; }
void inizializza_decrescente(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = dim-i; }

static int uv[N_MAX];
static int ov[N_MAX];

int main( void ) {
	int n;
	double t;
	srand(time(NULL));
	
	printf("Prestazioni dell'algoritmo INSERTION-SORT\n");

	printf("\n\nOrdinamento crescente di valori gia' ordinati in modo crescente (Best Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]     t/n      t/n^2\n");
	printf("----------------------------------------------------------------------\n");
	for(int n=2000000; n <= 4000000; n+=500000) {
		double t1, t2, t3, t;
		double k1, k2;
		inizializza_crescente(uv, n);
		t1 = misura_tempo(ov, uv, n);
		t2 = misura_tempo(ov, uv, n);
		t3 = misura_tempo(ov, uv, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf    %10.1e %10.1e\n",
				n, t1, t2, t3, t, k1, k2);
	}

	printf("\n\nOrdinamento crescente di valori casuali (Average Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]     t/n      t/n^2\n");
	printf("----------------------------------------------------------------------\n");
	for(int n=10000; n <= 25000; n+=5000) {
		double t1, t2, t3, t;
		double k1, k2;
		inizializza_casuale(uv, n);
		t1 = misura_tempo(ov, uv, n);
		t2 = misura_tempo(ov, uv, n);
		t3 = misura_tempo(ov, uv, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf    %10.1e %10.1e\n",
				n, t1, t2, t3, t, k1, k2);
	}

	printf("\n\nOrdinamento crescente di valori gia' ordinati in modo decrescente (Worst Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]     t/n      t/n^2\n");
	printf("----------------------------------------------------------------------\n");
	for(int n=10000; n <= 25000; n+=5000) {
		double t1, t2, t3, t;
		double k1, k2;
		inizializza_decrescente(uv, n);
		t1 = misura_tempo(ov, uv, n);
		t2 = misura_tempo(ov, uv, n);
		t3 = misura_tempo(ov, uv, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf    %10.1e %10.1e\n",
				n, t1, t2, t3, t, k1, k2);
	}

	return 0;
}


