/*
 *           Titolo: Studio delle prestazioni dell'algoritmo "QUICK SORT"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 22/11/2016
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>
#include <math.h>

#define N_MAX 500000

void quick_sort(int vett[], int inf, int sup) {
    int sx = inf;
    int dx = sup;
    int pivot;
    
    pivot = vett[(inf + sup) / 2];
    
    while (sx <= dx) {
        while (vett[sx] < pivot) sx++;
        while (vett[dx] > pivot) dx--;
        
        if (sx < dx) {
            int temp = vett[sx];
            vett[sx] = vett[dx];
            vett[dx] = temp;
        }
        if (sx <= dx) {
            sx++;
            dx--;
        }
    }
    
    if (inf < dx)
        quick_sort(vett, inf, dx);
       
    if (sx < sup)
        quick_sort(vett, sx, sup);
}

double misura_tempo(int vett[], int dim) {
	clock_t t_inizio, t_fine;
	clock_t ticks;

	t_inizio = clock();     // Tempo di CPU consumato dal processo fino a questo punto [tick]
	
	quick_sort(vett, 0, dim - 1);
	
	t_fine = clock(); 		// Nuovo tempo di CPU consumato dal processo fino a questo punto [tick]
	
	ticks = t_fine - t_inizio;  // Intervallo di tempo tra i due istanti [tick]
	
	return 1000.0 * ticks / CLOCKS_PER_SEC;     // Conversione dell'intervallo di tempo in ms
	
}

void inizializza_casuale(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = rand(); }

void inizializza_crescente(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = i+1; }

void inizializza_decrescente(int vett[], int dim) { 
    int c=1;
    for (int i=0; i < dim/2; i++)  {
        vett[i] = c++;
        vett[dim-1-i] = c++;
    }
 }


static int v[N_MAX];

void stampa_primi_10(int vett[]) {
	for (int i=0; i < 10; i++) printf(" %d", vett[i]);
	printf("\n");
}

int main( void ) {
	int n;
	double t;
	srand(time(NULL));
	
	printf("Prestazioni dell'algoritmo QUICK-SORT\n");

	printf("\n\nOrdinamento crescente di valori gia' ordinati in modo crescente (Best Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]   t/n  t/(n*log(n))  t/n^2\n");
	printf("----------------------------------------------------------------------------\n");
	for(int n=20000; n <= 50000; n+=10000) {
		double t1, t2, t3, t;
		double k1, k2, k3;
		inizializza_crescente(v, n);
		t1 = misura_tempo(v, n);
		inizializza_crescente(v, n);
		t2 = misura_tempo(v, n);
		inizializza_crescente(v, n);
		t3 = misura_tempo(v, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/(n*log(n)/log(2));
		k3 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf  %10.1e%10.1e%10.1e\n",
				n, t1, t2, t3, t, k1, k2, k3);
	}

	printf("\n\nOrdinamento crescente di valori casuali (Average Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]   t/n  t/(n*log(n))  t/n^2\n");
	printf("----------------------------------------------------------------------------\n");
	for(int n=20000; n <= 50000; n+=10000) {
		double t1, t2, t3, t;
		double k1, k2, k3;
		inizializza_casuale(v, n);
		t1 = misura_tempo(v, n);
		inizializza_casuale(v, n);
		t2 = misura_tempo(v, n);
		inizializza_casuale(v, n);
		t3 = misura_tempo(v, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/(n*log(n)/log(2));
		k3 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf  %10.1e%10.1e%10.1e\n",
				n, t1, t2, t3, t, k1, k2, k3);
	}

	printf("\n\nOrdinamento crescente di valori (Worst Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]   t/n  t/(n*log(n))  t/n^2\n");
	printf("----------------------------------------------------------------------------\n");
	for(int n=20000; n <= 50000; n+=10000) {
		double t1, t2, t3, t;
		double k1, k2, k3;
		inizializza_decrescente(v, n);
		t1 = misura_tempo(v, n);
		inizializza_decrescente(v, n);
		t2 = misura_tempo(v, n);
		inizializza_decrescente(v, n);
		t3 = misura_tempo(v, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/(n*log(n)/log(2));
		k3 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf  %10.1e%10.1e%10.1e\n",
				n, t1, t2, t3, t, k1, k2, k3);
	}

	return 0;

}


