/*
 *           Titolo: Studio delle prestazioni dell'algoritmo "SELECTION SORT"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 22/11/2016
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define N_MAX 250000

void selection_sort(int vett[], int dim) {
	for(int i = 0; i < dim; i++) {
		int i_min = i;
		for(int j = i+1; j < dim; j++) {
			if (vett[j] < vett[i_min])
				i_min = j;
		}

		if (i != i_min) {
			int temp = vett[i];
			vett[i] = vett[i_min];
			vett[i_min] = temp;
		}
	}
}

double misura_tempo(int vett[], int dim) {
	clock_t t_inizio, t_fine;
	clock_t ticks;

	t_inizio = clock();     // Tempo di CPU consumato dal processo fino a questo punto [tick]
	
	selection_sort(vett, dim);
	
	t_fine = clock(); 		// Nuovo tempo di CPU consumato dal processo fino a questo punto [tick]
	
	ticks = t_fine - t_inizio;  // Intervallo di tempo tra i due istanti [tick]
	
	return 1000.0 * ticks / CLOCKS_PER_SEC;     // Conversione dell'intervallo di tempo in ms
	
}

void inizializza_casuale(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = rand(); }

void inizializza_crescente(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = i+1; }

void inizializza_decrescente(int vett[], int dim) { for (int i=0; i < dim; i++) vett[i] = dim-i; }

static int v[N_MAX];

int main( void ) {
	int n;
	double t;
	srand(time(NULL));
	
	printf("Prestazioni dell'algoritmo SELECTION-SORT\n");

	printf("\n\nOrdinamento crescente di valori gia' ordinati in modo crescente (Best Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]     t/n      t/n^2\n");
	printf("----------------------------------------------------------------------\n");
	for(int n=10000; n <= 25000; n+=5000) {
		double t1, t2, t3, t;
		double k1, k2;
		inizializza_crescente(v, n);
		t1 = misura_tempo(v, n);
		inizializza_crescente(v, n);
		t2 = misura_tempo(v, n);
		inizializza_crescente(v, n);
		t3 = misura_tempo(v, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf    %10.1e %10.1e\n",
				n, t1, t2, t3, t, k1, k2);
	}

	printf("\n\nOrdinamento crescente di valori casuali (Average Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]     t/n      t/n^2\n");
	printf("----------------------------------------------------------------------\n");
	for(int n=10000; n <= 25000; n+=5000) {
		double t1, t2, t3, t;
		double k1, k2;
		inizializza_casuale(v, n);
		t1 = misura_tempo(v, n);
		inizializza_casuale(v, n);
		t2 = misura_tempo(v, n);
		inizializza_casuale(v, n);
		t3 = misura_tempo(v, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf    %10.1e %10.1e\n",
				n, t1, t2, t3, t, k1, k2);
	}

	printf("\n\nOrdinamento crescente di valori gia' ordinati in modo decrescente (Worst Case)\n");
	printf("      n            Tempi [ms]       t medio [ms]     t/n      t/n^2\n");
	printf("----------------------------------------------------------------------\n");
	for(int n=10000; n <= 25000; n+=5000) {
		double t1, t2, t3, t;
		double k1, k2;
		inizializza_decrescente(v, n);
		t1 = misura_tempo(v, n);
		inizializza_decrescente(v, n);
		t2 = misura_tempo(v, n);
		inizializza_decrescente(v, n);
		t3 = misura_tempo(v, n);
		t = (t1+t2+t3) / 3.0;
		k1 = t/n;
		k2 = t/n/n;
		printf("%9d %7.0lf %7.0lf %7.0lf    %7.1lf    %10.1e %10.1e\n",
				n, t1, t2, t3, t, k1, k2);
	}

	return 0;
}


